package edu.kit.ipd.descartes.adaptation.evaluation.performance.repository;

import static edu.kit.ipd.descartes.adaptation.evaluation.performance.repository.PerfDataRepoConverter.convertObservableImpact;

import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

import edu.kit.ipd.descartes.adaptation.model.AbstractEcoreModelLoader;
import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.Result;
import edu.kit.ipd.descartes.ql.models.repository.Observation;
import edu.kit.ipd.descartes.ql.models.repository.ObservationRepository;
import edu.kit.ipd.descartes.ql.models.repository.RepositoryFactory;
//import edu.kit.ipd.descartes.adaptation.model.repository.dmm.AdaptationProcessModelLoader;

public class ObservationRepositoryHandler implements IPerformanceDataRepositoryHandler {

    private static Logger logger = Logger.getLogger(ObservationRepositoryHandler.class);
    private AbstractEcoreModelLoader modelLoader = null;
    private ObservationRepository repository = null;

    public ObservationRepositoryHandler() {
        modelLoader = ObservationRepositoryModelLoader.getInstance();
        logger.debug("ObservationRepositoryHandler loaded.");
    }

    @Override
    public Impact getLatestImpact() {
        return convertObservableImpact(repository.getLatestImpact());
    }

    @Override
    public void load(String modelFileUri) {
        repository = (ObservationRepository) modelLoader.load(URI.createFileURI(modelFileUri));
    }

    @Override
    public Result getCurrentSystemState() {
        Observation o = repository.getLatestObservation();

        if (o == null)
            return null;
        else 
            return PerfDataRepoConverter.convertResult(o);
    }

    @Override
    public EObject getRepository() {
        return repository;
    }
    

	@Override
	public void setNewSystemState() {
		List<Observation> observations = repository.getObservations();
		Observation currentObservation = repository.getLatestObservation();
		Observation newObservation = null;
		for (Iterator<Observation> iterator = observations.iterator(); iterator.hasNext();) {
			Observation observ = (Observation) iterator.next();
			if (EcoreUtil.equals(observ, currentObservation)) {
				if (iterator.hasNext()) {
					newObservation = iterator.next();
				} else {
					// Just in case there is no remaining observation in the repository
					newObservation = RepositoryFactory.eINSTANCE.createObservation();
					newObservation.setIdentifier("EMPTY");
				}
				break;
			}
		}
		if (newObservation != null) {
			repository.setLatestObservation(newObservation);
		}
		else {
			logger.error("Error determining new system state!");
			throw new RuntimeException();
		}
	}
}
