package edu.kit.ipd.descartes.adaptation.evaluation.performance.repository;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;

import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.PerformanceDataRepository;
import edu.kit.ipd.descartes.perfdatarepo.Result;

public class PerformanceDataRepositoryHandler implements IPerformanceDataRepositoryHandler {
    
    private static Logger logger = Logger.getLogger(PerformanceDataRepositoryHandler.class); 
    
    private int iteration = 0;
    
    private PerformanceDataRepository repository = null;
    private PerformanceDataRepositoryModelLoader modelLoader = null;
    
    public PerformanceDataRepositoryHandler() {
        modelLoader = PerformanceDataRepositoryModelLoader.getInstance();
        logger.debug("PerformanceDataRepositoryHandler instantiated.");
    }
    
    public void load(String fileUri) {
        repository = modelLoader.load(URI.createFileURI(fileUri));
    }

    @Override
    public Impact getLatestImpact() {
        EList<Impact> impactHistory = repository.getImpactHistory();
        Impact mostRecentImpact =  impactHistory.get(impactHistory.size() - 1);
                
        for (Impact currentImpact : impactHistory) {
            Result afterResult = currentImpact.getAfter();
            if (afterResult.getTimestamp().compareTo(mostRecentImpact.getAfter().getTimestamp()) > 0 )
                mostRecentImpact = currentImpact;
        }
        
        return mostRecentImpact;
    }

    @Override
    public Result getCurrentSystemState() {
        return repository.getResultHistory().get(iteration++);
    }

    @Override
    public EObject getRepository() {
        return repository;
    }

	@Override
	public void setNewSystemState() {
		// TODO Auto-generated method stub
		
	}

}
