package edu.kit.ipd.descartes.adaptation.evaluation.performance.weightingfunction;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;

import edu.kit.ipd.descartes.mm.adaptation.WeightedTactic;
import edu.kit.ipd.descartes.mm.adaptation.WeightingFunction;
import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.MetricValue;
import edu.kit.ipd.descartes.perfdatarepo.Result;

public class WeightedSum implements IWeightingFunction {

	private Logger logger = Logger.getLogger(WeightedSum.class);
	private WeightingFunction weightingFunction = null;

	public WeightedSum() {
	}
	
    public WeightingFunction getWeightingFunction() {
        return weightingFunction;
    }

    @Override
    public void setWeightingFunction(WeightingFunction weightingFunction) {
        this.weightingFunction = weightingFunction;
    }

	@Override
	public double calculateWeight(WeightedTactic weightedtactic) {

		double newWeight = 0.0;
		Impact impact = weightedtactic.getLastImpact();
		Result before = impact.getBefore();
		Result after = impact.getAfter();
		EList<MetricValue> afterMetricValues = after.getMetricValues();

		for (MetricValue afterMetricValue : afterMetricValues) {
			// calculated delta=(after-before)
			double afterValue = afterMetricValue.getValue();
			double beforeValue = WeightedSumWeightingFunctionHelper
					.getValueForMetricType(afterMetricValue.getMetricType(), before);
			double delta = afterValue - beforeValue;

			// get the corresponding weight
			double weightOfMetric = WeightedSumWeightingFunctionHelper.getWeightForMetricType(weightingFunction,
					afterMetricValue.getMetricType());

            if (Double.isNaN(weightOfMetric))
                logger.debug("WeightingFunction "+ weightingFunction.getName()+" has no weight specified for metric type " + afterMetricValue.getMetricType().getName() + ". Skipping...");
            else {
                // add it
                newWeight += weightOfMetric * delta;
                logger.debug("Delta for metric type " + afterMetricValue.getMetricType().getName() + " was: " + delta
                        + " and weight is " + weightOfMetric);
            }
		}

		return newWeight;
	}

}
